library(shiny)
library(currentSurvival)

# load the data
#data(cml)
rm(list=ls())

# Define server logic required to plot current survival measures
shinyServer(function(input, output) {

  # Load data in a reactive expression:
  data <- reactive({
    # input$file1 will be NULL initially. After the user selects and uploads a 
    # file, it will be a data frame with 'name', 'size', 'type', and 'datapath' 
    # columns. The 'datapath' column will contain the local filenames where the 
    # data can be found.
    inFile <- input$file1
    if (is.null(inFile))
      return(NULL)
    read.table(inFile$datapath, header=input$header, sep=input$sep, dec=input$dec) 
  })

  # Calculate results in a reactive expression:
  res <- reactive({
    if (is.null(data())) {
      return(NULL)
    } else {
      if(input$measure=="cci"){
         if(input$confint==FALSE){
           cci(data(),fig=FALSE)
         } else {
           cci(data(),conf.int=TRUE,no.iter=input$ci_noiter,fig=FALSE)
         }
      } else {
         if(input$confint==FALSE){
           clfs(data(),fig=FALSE)
         } else {
           clfs(data(),conf.int=TRUE,no.iter=input$ci_noiter,fig=FALSE)
         }
      }
    }
  })

  # Compute the formula text in a reactive expression:
  formulaText <- reactive({
    paste("Figure - ", input$measure)
  })

  # Return the formula text for printing as a caption of a plot:
  output$caption <- renderText({
    if (is.null(data())) {
      return(NULL)
    } else {
      formulaText()
    }
  })

  # Figure:
  output$cursurvPlot <- renderPlot({
    if (is.null(data())) {
      return(NULL)
    } else {
      results <- res()
      maxx <- max(results$pest.day[,1]) # maximum follow-up time in days
      x=0:maxx
      yrs <- floor(maxx/365) # maximum follow-up time in years
      if(input$confint==FALSE){
        if(input$measure=="cci") {
          plot(0,0,pch='.',cex=0.01,xlab="Years after therapy initiation",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CCI","comCI"),lwd=c(1,1),lty=c(1,2),bty="n",cex=0.9)
        } else {
          plot(0,1,pch='.',cex=0.01,xlab="Years after achievement of the first disease remission",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CLFS","LFS"),lwd=c(1,1),lty=c(1,2),bty="n",cex=0.9)
        }
        axis(2,at=seq(0,1,0.2)) # setting of points where tick-marks are to be drawn on the y-axis
        axis(1,at=seq(0,((yrs+1)*365),365),labels=seq(0,(yrs+1),1)) # setting of points where tick-marks are to be drawn on the x-axis
        lines(x,results$pest.day[,2],type="S",lty=1,lwd=1) # CCI or CLFS estimate
        lines(x,results$pest.day[,3],type="S",lty=2,lwd=1) # comCCI or LFS estimate
      } else {
        if(input$measure=="cci") {
          plot(0,0,pch='.',cex=0.01,xlab="Years after therapy initiation",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CCI","95% conf. int.","comCI","95% conf. int."),lwd=c(2,1,2,1),lty=c(1,1,2,2),bty="n",cex=0.9)
        } else {
          plot(0,1,pch='.',cex=0.01,xlab="Years after achievement of the first disease remission",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CLFS","95% conf. int.","LFS","95% conf. int."),lwd=c(2,1,2,1),lty=c(1,1,2,2),bty="n",cex=0.9)
        }
        axis(2,at=seq(0,1,0.2)) # setting of points where tick-marks are to be drawn on the y-axis
        axis(1,at=seq(0,((yrs+1)*365),365),labels=seq(0,(yrs+1),1)) # setting of points where tick-marks are to be drawn on the x-axis
        lines(x,results$pest.day[,2],type="S",lty=1,lwd=1) # lower confidence interval for the CCI or CLFS function estimate
        lines(x,results$pest.day[,3],type="S",lty=1,lwd=2) # CCI or CLFS estimate
        lines(x,results$pest.day[,4],type="S",lty=1,lwd=1) # upper confidence interval for the CCI or CLFS function estimate
        lines(x,results$pest.day[,5],type="S",lty=2,lwd=1) # lower confidence interval for the comCI or LFS function estimate
        lines(x,results$pest.day[,6],type="S",lty=2,lwd=2) # comCI or LFS estimate
        lines(x,results$pest.day[,7],type="S",lty=2,lwd=1) # upper confidence interval for the comCI or LFS function estimate
      }
    }
  })

  # Compute the formula text in a reactive expression:
  formulaText2 <- reactive({
    paste("Table - ", input$measure)
  })

  # Return the formula text for printing as a caption of a plot:
  output$caption2 <- renderText({
    if (is.null(data())) {
      return(NULL)
    } else {
      formulaText2()
    }
  })


  # Table with estimates:
  output$view <- renderTable({
    if (is.null(data())) {
      return(NULL)
    } else {
      results <- res()
      results$pest
    }
  })

  # Download table:
  output$downloadTable <- downloadHandler(
    filename = function() { paste('Table_',input$measure,'.txt',sep='') },
    content = function(file) {
      if (is.null(data())) {
        return(NULL)
      } else {
        results <- res()
      }
      write.table(results$pest, file, sep="\t", dec=".")
  })

  # Download figure:
  output$downloadFigure <- downloadHandler(
    filename = function() { 
      if(input$figtype=="pdf") { 
        paste('Figure_',input$measure,'.pdf',sep='')
      } else {
        paste('Figure_',input$measure,'.png',sep='')
      }
    },
    content = function(file) {
    if (is.null(data())) {
      return(NULL)
    } else {
      results <- res()
      maxx <- max(results$pest.day[,1]) # maximum follow-up time in days
      x=0:maxx
      yrs <- floor(maxx/365) # maximum follow-up time in years
      if(input$figtype=="pdf") {
         pdf(file,width=6,height=6)
      } else {
         png(file,width=400,height=400)
      }
      if(input$confint==FALSE){
        if(input$measure=="cci") {
          plot(0,0,pch='.',cex=0.01,xlab="Years after therapy initiation",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CCI","comCI"),lwd=c(1,1),lty=c(1,2),bty="n",cex=0.9)
        } else {
          plot(0,1,pch='.',cex=0.01,xlab="Years after achievement of the first disease remission",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CLFS","LFS"),lwd=c(1,1),lty=c(1,2),bty="n",cex=0.9)
        }
        axis(2,at=seq(0,1,0.2)) # setting of points where tick-marks are to be drawn on the y-axis
        axis(1,at=seq(0,((yrs+1)*365),365),labels=seq(0,(yrs+1),1)) # setting of points where tick-marks are to be drawn on the x-axis
        lines(x,results$pest.day[,2],type="S",lty=1,lwd=1) # CCI or CLFS estimate
        lines(x,results$pest.day[,3],type="S",lty=2,lwd=1) # comCCI or LFS estimate
      } else {
        if(input$measure=="cci") {
          plot(0,0,pch='.',cex=0.01,xlab="Years after therapy initiation",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CCI","95% conf. int.","comCI","95% conf. int."),lwd=c(2,1,2,1),lty=c(1,1,2,2),bty="n",cex=0.9)
        } else {
          plot(0,1,pch='.',cex=0.01,xlab="Years after achievement of the first disease remission",ylab="Probability",axes=FALSE,xlim=c(0,maxx),ylim=c(0,1)) # plot initialization
          legend("bottomright",legend=c("CLFS","95% conf. int.","LFS","95% conf. int."),lwd=c(2,1,2,1),lty=c(1,1,2,2),bty="n",cex=0.9)
        }
        axis(2,at=seq(0,1,0.2)) # setting of points where tick-marks are to be drawn on the y-axis
        axis(1,at=seq(0,((yrs+1)*365),365),labels=seq(0,(yrs+1),1)) # setting of points where tick-marks are to be drawn on the x-axis
        lines(x,results$pest.day[,2],type="S",lty=1,lwd=1) # lower confidence interval for the CCI or CLFS function estimate
        lines(x,results$pest.day[,3],type="S",lty=1,lwd=2) # CCI or CLFS estimate
        lines(x,results$pest.day[,4],type="S",lty=1,lwd=1) # upper confidence interval for the CCI or CLFS function estimate
        lines(x,results$pest.day[,5],type="S",lty=2,lwd=1) # lower confidence interval for the comCI or LFS function estimate
        lines(x,results$pest.day[,6],type="S",lty=2,lwd=2) # comCI or LFS estimate
        lines(x,results$pest.day[,7],type="S",lty=2,lwd=1) # upper confidence interval for the comCI or LFS function estimate
      }
      dev.off()
    }
  })

})

